/*
	StringControl.h
	© 1997 Vikram Kulkarni (vkulkarn@uiuc.edu)
	11/6/97 - Wrote it
	11/7/97 - Added comments

	To see what a string control does
	look at the TestStringControl app
	that should have come with this.
                                        */


#ifndef _STRING_CONTROL_H_
#define _STRING_CONTROL_H_

#include <Control.h>
#include <string.h>

class StringControl : public BControl
{
	public:
		StringControl	(// passed to the BView constructor
						 BRect frame,
						 // passed to the BView constructor
						 char * name,
						 // the text that's shown
						 char * label,
						 // the message that's sent
						 BMessage * message,
						 // the text color when the mouse is over the control
						 // OR when the control is the Focus view
						 // A blueish color by default
						 const rgb_color * glow_color = NULL,
						 // the text color at other times
						 // black by default
						 const rgb_color * text_color = NULL,
						 // the font
						 // be_plain_font by default
						 const BFont * font = NULL,
						 // passed to the BView constructor
						 uint32 resizingMode = B_FOLLOW_LEFT | B_FOLLOW_TOP,
						 // passed to the BView constructor
						 uint32 flags = B_WILL_DRAW | B_NAVIGABLE);
		
		// Cleans up
		~StringControl();
		// Draw's the text
		virtual void Draw(BRect updateRect);
		// Makes the text glow
		virtual void MakeFocus(bool focused = true);
		// sends the message
		virtual void MouseDown(BPoint where);
		// makes the text glow if the mouse is over the view
		// UNLESS the user is dragging something (b/c it doesn't
		// know what to do with the dragged message)
		virtual void MouseMoved(BPoint point,
								 uint32 transit,
								 const BMessage * message);
		// sets width and height to the preferred dimensions
		// BUT does not change the view size
		virtual void GetPreferredSize(float * width, float * height);
		// resizes the view to the perferred size
		// the top left corner stays the same
		virtual void ResizeToPreferred();
		// if the control is disabled it wont glow
		virtual void SetEnabled( bool enabled);
		
		// Changes the text color
		void SetTextColor(rgb_color new_text_color);
		void SetTextColor(uint8 red, uint8 green, uint8 blue, uint8 alpha = 0);
		// returns the text color
		rgb_color TextColor() const ;

		// Changes the glow color
		void SetGlowColor(rgb_color new_glow_color);
		void SetGlowColor(uint8 red, uint8 green, uint8 blue, uint8 alpha = 0);
		// returns the glow color
		rgb_color GlowColor() const ;
			
	private:
		BFont			* ffont; // the font
		BPoint			  draw_point; // used to vertically align the text
		rgb_color		* ftext_color; // the text color
		rgb_color		* fglow_color; // the glow colot
		bool			  fenabled; // is the control enabled
		bool			  fglowing; // is it glowing
		bool			  ffocused; // is it the focus view
};

#endif