// Beph 1.1
// Vikram Kulkarni
// BephNet.cpp

#include "BephNet.h"
#include <bsd_mem.h>

#define MAXLINE 1024

#define SERVER_REPLY 'srpl'
#define CONNECT 'cnct'
#define CONNECTED_TO_SERVER 'conn'
#define CONNECT_FAILED 'cnfl'
#define NO_NAME_SERVER 'nnsr'
#define QUERY_DONE 'done'
#define TEST 'test'
#define QUERY 'qery'
#define NO_MATCH 'nomc'

/* ---------- CLASS BephNet ---------- */
/* -----BephNet::BephNet ----- */

BephNet::BephNet(char *Server,BWindow *Window, BLooper *BephParser)
	:BLooper("BephNet")
{
	server = new char[strlen(Server) + 1];
	strcpy(server,Server);

	win = Window;
	parser = BephParser;
	
	phPort = 105;
	
	Run();
}

/* ----- BephNet::MessageReceived ----- */
void BephNet::MessageReceived(BMessage *msg)
{
	switch(msg->what)
	{
		case CONNECT:
		{
			ConnectToServer();
			break;
		}
		case QUERY:
		{
			Query(msg->FindString("PH_COMMAND"), msg->FindBool("RETURN_ALL_STATE"), msg->FindBool("ALIAS_EQUALS"));
			break;
		}
		
		default:
		{
			BLooper::MessageReceived(msg);
			break;
		}
	}
}

/* ----- BephNet::~BephNet ----- */
BephNet::~BephNet()
{
	if (phsocket)
	{
		send(phsocket, "quit\n", 5, 0);
		closesocket(phsocket);
	}
}

/* ----- BephNet::ConnectToServer ----- */
int BephNet::ConnectToServer()
{
	struct sockaddr_in sa;
	struct hostent *hp;
	
	bzero(&sa, sizeof(sa));
	
	hp = gethostbyname(server);
	
	if (hp == NULL)
	{
		win->PostMessage(NO_NAME_SERVER);
		return(0);
	}

	bcopy(hp->h_addr, (char *) &sa.sin_addr, hp->h_length);
	
	memset(sa.sin_zero, 0, sizeof(sa.sin_zero));
	
	sa.sin_family = AF_INET;
	sa.sin_port = htons((u_short)phPort);
	
	if ((phsocket = socket(AF_INET, SOCK_STREAM, 0)) < 0)
		return (0);

	if (connect(phsocket, (struct sockaddr *) &sa, sizeof(sa)) < 0)
	{
		win->PostMessage(CONNECT_FAILED);
		closesocket(phsocket);
		return (0);
	}

	win->PostMessage(CONNECTED_TO_SERVER);
	return (phsocket);
}

/* ----- BephNet::Query ----- */
void BephNet::Query(const char *command, const bool return_all, const bool alias_equals)
{
	char *tmpLineBuf = new char[MAXLINE];
	bool first = true;
	
	BMessage *msg = new BMessage(SERVER_REPLY);
	msg->AddString("SERVER_REPLY_LINE", "krap");
	
	if ((command[0] == 'h') && (command[1] == 'e') && (command[2] == 'l') && (command[3] == 'p'))
	{
		if (strlen(command) != 4)
		{
			send(phsocket, command, strlen(command), 0);
			send(phsocket, "\r\n", 2, 0);
		}
		else
		{
			send(phsocket, "help ph\r\n", 9, 0);
		}
	}
	else
	{
		send(phsocket, "query ", 6, 0);
		if (alias_equals) { send(phsocket, " alias = ", 9, 0); }
		send(phsocket, command, strlen(command), 0);
		if (return_all) { send(phsocket, " return all", 11, 0); }
		send(phsocket, "\r\n", 2, 0);
	}
	
	while (1)
	{
		for (int i = 0; i < MAXLINE - 1; i++)
		{
			recv(phsocket, &tmpLineBuf[i], 1, 0);
			if (tmpLineBuf[i] == '\n')
			{
				tmpLineBuf[++i] = '\0';
				break;
			}
		}
		
		msg->ReplaceString("SERVER_REPLY_LINE", tmpLineBuf);
		parser->PostMessage(msg);
		
		if ((tmpLineBuf[0] == '5') || (tmpLineBuf[0] == '2'))
			break;
		
		first = true;
	}

	parser->PostMessage(QUERY_DONE);

	if (!strncmp(tmpLineBuf, "501:No matches to your query.", 14))
	{
		BMessage *no_match_msg = new BMessage(NO_MATCH);
		no_match_msg->AddString("COMMAND", command);
		parser->PostMessage(no_match_msg);
		delete (no_match_msg);
	}

	delete (tmpLineBuf);
	delete (msg);
}
