// Beph 1.1
// Vikram Kulkarni
// BephParser.cpp

#include "BephParser.h"

#define SERVER_REPLY 'srpl'
#define QUERY_DONE 'done'
#define NEXT_MATCH 'nxtm'
#define NO_MATCH 'nomc'
#define HOW_MANY_MATCHES 'hmny'
#define TEST 'test'

/* ---------- CLASS BephParser ---------- */
/* ----- BephParser::BephParser ----- */
BephParser::BephParser(BWindow *Window)
	:BLooper ("BephParser")
{
	win = Window;
	
	line_number = 1;
	last_reply_number = 0;
	
	Run(); 
}

/* ----- BephParser::MessageReceived ----- */
void BephParser::MessageReceived(BMessage *msg)
{
	switch(msg->what)
	{
		case SERVER_REPLY:
			{
				char *parsed;
				parsed = RemovePhCodes(msg->FindString("SERVER_REPLY_LINE"));
				
				BMessage *new_msg = new BMessage(SERVER_REPLY);
				new_msg->AddString("SERVER_REPLY_LINE", parsed);
				
				win->PostMessage(new_msg);
				break;
			}
		
		case QUERY_DONE:
			{
				line_number = 1;
				last_reply_number = 0;
				win->PostMessage(QUERY_DONE);
				break;
			}
			
		case NO_MATCH:
			{
				win->PostMessage(msg);
				break;
			}
			
		default:
			{
				BLooper::MessageReceived(msg);
				break;
			}
	}
}

/* ----- BephParser::RemovePhCodes ----- */
char *BephParser::RemovePhCodes(const char *unparsed)
{
	int server_code = -1;
	int reply_number = -1;
	char field_name[512] = {'\0'};
	char field_value[512] = {'\0'};
	int error;
	error = sscanf(unparsed, "-%i:%i:%[^:]:%[^\n]",&server_code, &reply_number, field_name, field_value);

	if(error == 4)
	{// This line contains an actual reply, not a server message
		if (reply_number != last_reply_number)
		{
			last_reply_number = reply_number;
			win->PostMessage(NEXT_MATCH);
		}
		char temp_line[1024];
		sprintf(temp_line, "%20.20s: %s\n", field_name, field_value);
		return strdup(temp_line);
	}
	else
	{// This line contains a server message
		error = sscanf(unparsed, "%i:%[^.]",&server_code, field_value);
		if (error == 2)
		{
			switch(server_code)
			{
				case 102:
				{
					int how_many_matches = -1;
					sscanf(field_value, "%*[^0987654321]%i", &how_many_matches);
					BMessage msg(HOW_MANY_MATCHES);
					msg.AddInt32("HowMany", how_many_matches);
					win->PostMessage(&msg);
					break;
				}
				case 200:
				{//reached last message... a "-------" will look good....
					win->PostMessage(NEXT_MATCH);
					break;
				}
			}
			
			sprintf(field_name,"%s\n", field_value);
			return strdup(field_name);
		}
		
		return strdup("\n");;
	}
}